"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const discord_js_1 = require("discord.js");
const CommandLoader_1 = require("../command/CommandLoader");
const CommandRegistry_1 = require("../command/CommandRegistry");
const EventLoader_1 = require("../event/EventLoader");
const EventRegistry_1 = require("../event/EventRegistry");
class SilentClient extends discord_js_1.Client {
    constructor(config, clientConfig) {
        super(clientConfig);
        this.base = true;
        this.commandDir = config.commandDir;
        this.ownerID = config.owner;
        this._token = config.token;
        this.eventDir = config.eventDir;
        this.prefix = config.prefix;
        this.notFoundError = config.notFoundError || null;
        this.baseDir = config.baseDir || require(`${process.cwd()}/tsconfig.json`).outDir || '';
        this.base = config.base || true;
        this.commands = new CommandRegistry_1.CommandRegistry(this);
        this._commandLoader = new CommandLoader_1.CommandLoader(this);
        this.events = new EventRegistry_1.EventRegistry(this);
        this._eventLoader = new EventLoader_1.EventLoader(this);
        this._eventLoader.loadFromPath(this.eventDir);
        this._commandLoader.loadFromPath(this.commandDir);
        if (this.base)
            this._commandLoader.loadFromPath(null, true);
        this.on('message', this.messageHandle);
    }
    get owner() {
        return this.ownerID;
    }
    start() {
        this.login(this._token);
        return this;
    }
    messageHandle(msg) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.events.get('message'))
                this.events.get('message').run(msg);
            if (!msg.content.startsWith(this.prefix) || msg.author.bot)
                return;
            const args = msg.content.slice(this.prefix.length).trim().split(/ +/g);
            const command = args.shift().toLowerCase();
            const resolvedCommand = this.commands.get(command) ||
                this.commands.find(rC => rC.aliases.includes(command));
            if (!resolvedCommand && this.notFoundError)
                return msg.reply(this.notFoundError);
            const userRestricted = [];
            const roleRestricted = [];
            resolvedCommand.restricted.forEach((value) => {
                if (value.type === 'user')
                    userRestricted.push(value.id);
                if (value.type === 'role')
                    roleRestricted.push(value.id);
            });
            if ((userRestricted.length && !userRestricted.includes(msg.author.id)) ||
                (roleRestricted.length && msg.member.roles.filter(role => roleRestricted.includes(role.id)).size === 0)) {
                return msg.reply('You do not have the permission to use this command');
            }
            if (resolvedCommand.ownerOnly && msg.author.id !== this.owner) {
                return msg.reply('You do not have the permission to use this command');
            }
            if (resolvedCommand.guildOnly && !msg.guild) {
                return msg.reply('This command can only be used in a guild.');
            }
            if (resolvedCommand.dmOnly && msg.guild) {
                return msg.reply('This command can only be used in DMs.');
            }
            if (resolvedCommand.nsfw && msg.guild && !msg.channel.nsfw) {
                return msg.reply('This command can only be used in a NSFW marked channel.');
            }
            if (resolvedCommand && msg.guild)
                resolvedCommand.run(msg, args);
        });
    }
}
exports.SilentClient = SilentClient;
//# sourceMappingURL=SilentClient.js.map